/*
    @library        : rlib
    @docs           : https://docs.rlib.io

    IF YOU HAVE NOT DIRECTLY RECEIVED THESE FILES FROM THE DEVELOPER, PLEASE CONTACT THE DEVELOPER
    LISTED ABOVE.

    THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS PUBLIC LICENSE
    ('CCPL' OR 'LICENSE'). THE WORK IS PROTECTED BY COPYRIGHT AND/OR OTHER APPLICABLE LAW. ANY USE OF
    THE WORK OTHER THAN AS AUTHORIZED UNDER THIS LICENSE OR COPYRIGHT LAW IS PROHIBITED.

    BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE BOUND BY THE TERMS
    OF THIS LICENSE. TO THE EXTENT THIS LICENSE MAY BE CONSIDERED TO BE A CONTRACT, THE LICENSOR GRANTS
    YOU THE RIGHTS CONTAINED HERE IN CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

    UNLESS OTHERWISE MUTUALLY AGREED TO BY THE PARTIES IN WRITING, LICENSOR OFFERS THE WORK AS-IS AND
    ONLY TO THE EXTENT OF ANY RIGHTS HELD IN THE LICENSED WORK BY THE LICENSOR. THE LICENSOR MAKES NO
    REPRESENTATIONS OR WARRANTIES OF ANY KIND CONCERNING THE WORK, EXPRESS, IMPLIED, STATUTORY OR
    OTHERWISE, INCLUDING, WITHOUT LIMITATION, WARRANTIES OF TITLE, MARKETABILITY, MERCHANTIBILITY,
    FITNESS FOR A PARTICULAR PURPOSE, NONINFRINGEMENT, OR THE ABSENCE OF LATENT OR OTHER DEFECTS, ACCURACY,
    OR THE PRESENCE OF ABSENCE OF ERRORS, WHETHER OR NOT DISCOVERABLE. SOME JURISDICTIONS DO NOT ALLOW THE
    EXCLUSION OF IMPLIED WARRANTIES, SO SUCH EXCLUSION MAY NOT APPLY TO YOU.
*/

/*
    library
*/

local base                  = rlib
local helper                = base.h
local design                = base.d
local ui                    = base.i

/*
*   loalization > misc
*/

local cfg                   = base.settings

/*
*   PANEL
*/

local PANEL = { }

/*
*   AccessorFunc
*/

AccessorFunc( PANEL, 'm_HideButtons', 'HideButtons' )

/*
*    init
*/

function PANEL:Init( )

    /*
    *   declarations
    */

    self.Offset                     = 0
    self.Scroll                     = 0
    self.CanvasSize                 = 1
    self.BarSize                    = 1

    /*
    *   btn > up
    */

    self.btnUp                      = ui.new( 'btn', self                   )
    :bsetup                         (                                       )
    :nodraw                         (                                       )

                                    :oc( function( s )
                                        s:GetParent( ):AddScroll( -1 )
                                    end )

    /*
    *   btn > down
    */

    self.btnDown                    = ui.new( 'btn', self                   )
    :bsetup                         (                                       )
    :nodraw                         (                                       )

                                    :oc( function( s )
                                        s:GetParent( ):AddScroll( 1 )
                                    end )

    /*
    *   btn > grip
    */

    self.btnGrip                    = ui.new( 'grip', self                  )

                                    :draw( function( s, w, h )
                                        local knobcolor = self.KnobColor or Color( 76, 158, 73 )
                                        design.rbox( 4, 0, 0, 20, h, Color( knobcolor.r, knobcolor.g, knobcolor.b, self.Alpha ) )
                                    end )

    /*
    *   parent
    */

    self                            = ui.get( self                          )
    :size                           ( 15                                    )
    :param                          ( 'SetHideButtons', false               )

                                    :draw( function( s, w, h )
                                        design.rbox( 4, 12, 10, 4, h - 20, Color( 24, 24, 24, self.Alpha ) )
                                    end )

end

/*
*   set enabled
*
*   @param  : bool b
*/

function PANEL:SetEnabled( b )
    if not b then
        self.Offset         = 0
        self:SetScroll      ( 0 )
        self.HasChanged     = true
    end

    self:SetMouseInputEnabled( b )
    self:SetVisible( b )
    if self.Enabled ~= b then
        self:GetParent( ):InvalidateLayout( )

        if self:GetParent( ).OnScrollbarAppear then
            self:GetParent( ):OnScrollbarAppear( )
        end
    end

    self.Enabled = b
end

/*
*   value
*/

function PANEL:Value( )
    return self.Pos
end

/*
*   bar scale
*/

function PANEL:BarScale( )
    if self.BarSize == 0 then return 1 end
    return self.BarSize / ( self.CanvasSize + self.BarSize )
end

/*
*   set up
*
*   @param  : int sz_bar
*   @param  : int sz_canvas
*/

function PANEL:SetUp( sz_bar, sz_canvas )
    self.BarSize            = sz_bar
    self.CanvasSize         = math.max( sz_canvas - sz_bar, 1 )

    self:SetEnabled         ( sz_canvas > sz_bar )
    self:InvalidateLayout   ( )
end

/*
*   on mouse wheeled
*
*   @param  : int dlta
*/

function PANEL:OnMouseWheeled( dlta )
    if not self:IsVisible( ) then return false end

    return self:AddScroll( dlta * -2 )
end

/*
*   add scroll
*
*   @param  : int dlta
*/

function PANEL:AddScroll( dlta )
    local OldScroll         = self:GetScroll( )
    dlta                    = dlta * 25
    self:SetScroll          ( self:GetScroll( ) + dlta )

    return OldScroll ~= self:GetScroll( )
end

/*
*   set scroll
*
*   @param  : int scrll
*/

function PANEL:SetScroll( scrll )
    if not self.Enabled then self.Scroll = 0 return end

    self.Scroll = math.Clamp( scrll, 0, self.CanvasSize )
    self:InvalidateLayout( )

    local func = self:GetParent( ).OnVScroll
    if func then
        func( self:GetParent( ), self:GetOffset( ) )
    else
        self:GetParent( ):InvalidateLayout( )
    end
end

/*
*   animate to
*
*   @param  : int scrll
*   @param  : int len
*   @param  : int delay
*   @param  : int ease
*/

function PANEL:AnimateTo( scrll, len, delay, ease )
    local anim              = self:NewAnimation( len, delay, ease )
    anim.StartPos           = self.Scroll
    anim.TargetPos          = scrll
    anim.Think = function( anim, pnl, fraction )
        pnl:SetScroll( Lerp( fraction, anim.StartPos, anim.TargetPos ) )
    end
end

/*
*   get scroll
*/

function PANEL:GetScroll( )
    if not self.Enabled then self.Scroll = 0 end
    return self.Scroll
end

/*
*   get offset
*/

function PANEL:GetOffset( )
    if not self.Enabled then return 0 end
    return self.Scroll * -1
end

/*
*   think
*/

function PANEL:Think( )
    if self.AlphaOR then
        self.Alpha          = 255
    end
end

/*
*   mouse pressed
*/

function PANEL:OnMousePressed( )
    local _, y              = self:CursorPos( )
    local sz_page           = self.BarSize

    if y > self.btnGrip.y then
        self:SetScroll      ( self:GetScroll( ) + sz_page )
    else
        self:SetScroll      ( self:GetScroll( ) - sz_page )
    end
end

/*
*   mouse released
*/

function PANEL:OnMouseReleased( )
    self.Dragging           = false
    self.DraggingCanvas     = nil
    self:MouseCapture       ( false )
    self.btnGrip.Depressed  = false
end

/*
*   cursor moved
*
*   @param  : int x
*   @param  : int y
*/

function PANEL:OnCursorMoved( x, y )
    if not self.Enabled then return end
    if not self.Dragging then return end

    local x, y              = self:ScreenToLocal( 0, gui.MouseY( ) )
    y                       = y - self.btnUp:GetTall( )
    y                       = y - self.HoldPos

    local sz_btn_h          = self:GetWide( )

    if self:GetHideButtons( ) then sz_btn_h = 0 end

    local track_size        = self:GetTall( ) - sz_btn_h * 2 - self.btnGrip:GetTall( )
    y                       = y / track_size

    self:SetScroll          ( y * self.CanvasSize )
end

/*
*   grip
*/

function PANEL:Grip( )
    if not self.Enabled then return end
    if self.BarSize == 0 then return end

    self:MouseCapture       ( true )
    self.Dragging           = true

    local _, y              = self.btnGrip:ScreenToLocal( 0, gui.MouseY( ) )
    self.HoldPos            = y

    self.btnGrip.Depressed  = true
end

/*
*   perform layout
*/

function PANEL:PerformLayout( )

    local sz_wide                   = self:GetWide( )
    local sz_btn_h                  = sz_wide

    if self:GetHideButtons( ) then sz_btn_h = 0 end

    local Scroll                    = self:GetScroll( ) / self.CanvasSize
    local BarSize                   = 20
    local track                     = self:GetTall( ) - ( sz_btn_h * 2 ) - BarSize
    track                           = track + 1
    Scroll                          = Scroll * track

    self.btnGrip:SetPos             ( 4, sz_btn_h + Scroll )
    self.btnGrip:SetSize            ( sz_wide, BarSize )

    if ( sz_btn_h > 0 ) then
        self.btnUp:SetPos           ( 4, 0, sz_wide, sz_wide )
        self.btnUp:SetSize          ( sz_wide, sz_btn_h )

        self.btnDown:SetPos         ( 4, self:GetTall( ) - sz_btn_h )
        self.btnDown:SetSize        ( sz_wide, sz_btn_h )

        self.btnUp:SetVisible       ( true )
        self.btnDown:SetVisible     ( true )
    else
        self.btnUp:SetVisible       ( false )
        self.btnDown:SetVisible     ( false )
        self.btnDown:SetSize        ( sz_wide, sz_btn_h )
        self.btnUp:SetSize          ( sz_wide, sz_btn_h )
    end

end

/*
*   paint
*
*   @param  : int w
*   @param  : int h
*/

function PANEL:Paint( w, h )
    derma.SkinHook( 'Paint', 'VScrollBar', self, w, h )
    return true
end

/*
*   register
*/

derma.DefineControl( 'rlib.ui.scrollbar', 'rlib scrollbar', PANEL, 'Panel' )
derma.DefineControl( 'rlib.ui.sbar.v1', 'rlib scrollbar v1', PANEL, 'DPanel' )